package crystalpalace.coff;

import crystalpalace.util.*;
import java.util.*;

public class Relocation extends COFFPart {
	/* The 32-bit relative address from the byte following the relocation. */
	public static final int IMAGE_REL_AMD64_REL32 = 0x04;
	public static final int IMAGE_REL_I386_DIR32  = 0x06;

	protected long   VirtualAddress;
	protected long   SymbolTableIndex;
	protected int    Type;

	public Relocation(COFFObject o, ByteWalker walker) {
		super(o);

		VirtualAddress       = walker.readInt();
		SymbolTableIndex     = walker.readInt();
		Type                 = walker.readShort();
	}

	public boolean is(String _machine, int _type) {
		return _machine.equals( getObject().getMachine() ) && Type == _type;
	}

	public Symbol getSymbol() {
		return getObject().getSymbol((int)SymbolTableIndex);
	}

	public String getSymbolName() {
		return getSymbol().getName();
	}

	public int getType() {
		return Type;
	}

	public long getVirtualAddress() {
		return VirtualAddress;
	}

	public void toString(String prefix, StringBuffer s) {
		s.append(prefix + "VirtualAddress:       " + CrystalUtils.toHex(VirtualAddress) + "\n");
		s.append(prefix + "SymbolTableIndex:     " + SymbolTableIndex + " (" + getSymbol().getName() + ")\n");
		s.append(prefix + "Type:                 " + Type + "\n");
	}

	public String toString() {
		return "Relocation(" + Type + ") " + getSymbolName() + " @ " + CrystalUtils.toHex(getVirtualAddress());
	}

	public String toSimpleString() {
		return getSymbolName() + " @ " + CrystalUtils.toHex(getVirtualAddress());
	}
}
