package crystalpalace.coff;

import crystalpalace.util.*;

public class SectionFlags extends COFFPart {
	protected long flags;

	public static final long IMAGE_SCN_CNT_CODE               = 0x00000020L;
	public static final long IMAGE_SCN_CNT_INITIALIZED_DATA   = 0x00000040L;
	public static final long IMAGE_SCN_CNT_UNINITIALIZED_DATA = 0x00000080L;
	public static final long IMAGE_SCN_MEM_EXECUTE            = 0x20000000L;
	public static final long IMAGE_SCN_MEM_READ               = 0x40000000L;
	public static final long IMAGE_SCN_MEM_WRITE              = 0x80000000L;

	public SectionFlags(COFFObject o, ByteWalker walker) {
		super(o);

		flags = walker.readInt();
	}

	public boolean flag(long x) {
		return (flags & x) == x;
	}

	public boolean isRead() {
		return flag(IMAGE_SCN_MEM_READ);
	}

	public boolean isWrite() {
		return flag(IMAGE_SCN_MEM_WRITE);
	}

	public boolean isExecute() {
		return flag(IMAGE_SCN_MEM_EXECUTE);
	}

	public boolean isInitialized() {
		return flag(IMAGE_SCN_CNT_INITIALIZED_DATA);
	}

	public boolean isUninitialized() {
		return flag(IMAGE_SCN_CNT_UNINITIALIZED_DATA);
	}

	public String toString() {
		StringBuffer s = new StringBuffer();

		s.append(isRead()    ? "r" : "-");
		s.append(isWrite()   ? "w" : "-");
		s.append(isExecute() ? "x" : "-");

		s.append(" ");

		if (flag(IMAGE_SCN_CNT_INITIALIZED_DATA)) {
			s.append("(init)");
			s.append(" ");
		}

		if (flag(IMAGE_SCN_CNT_UNINITIALIZED_DATA)) {
			s.append("(not init)");
			s.append(" ");
		}

		s.append(CrystalUtils.toHex(flags));

		return s.toString();
	}
}
