package crystalpalace.coff;

import java.util.*;
import crystalpalace.util.*;

     /* immediately following the symbol table, we have... the string table */
                /* See: https://courses.cs.washington.edu/courses/cse378/03wi/lectures/LinkerFiles/coff.pdf, pg. 41 */
public class StringTable extends COFFPart {
	protected int    length;
	protected byte[] data;

	public StringTable(COFFObject o, ByteWalker walker) {
		super(o);

		/*
		 * "At the beginning of the COFF string table are 4 bytes containing the total size (in bytes) of the rest
		 *  of the string table. This size includes the size field itself, so that the value in this location would be
		 *  4 if no strings were present.
		 *
		 *  Following the size are null-terminated strings pointed to by symbols in the COFF symbol table."
		 *
		 *  Part 5.6 of https://courses.cs.washington.edu/courses/cse378/03wi/lectures/LinkerFiles/coff.pdf
		 */
		length = walker.readInt();
		data   = walker.popBytes(length - 4);
	}

	/* internally, we reference strings at 0 starting AFTER the 4b length value.
	   but, in the COFF, strings are referenced at 0 starting BEFORE the 4b length value */
	public String getStringAt(int index) {
		return _getStringAt(index - 4);
	}

	protected String _getStringAt(int index) {
		try {
			int len = index;
			while (len < data.length) {
				if (data[len] == '\0')
					break;

				len++;
			}

			return new String(data, index, len - index, "UTF-8");
		}
		catch (Exception ex) {
			CrystalUtils.handleException(ex);
			return "";
		}
	}

	public void toString(String prefix, StringBuffer s) {
		s.append(prefix + "Length:               " + length + "\n");

		for (int x = 0; x < data.length;) {
			String temp = _getStringAt(x);
			if (temp.endsWith((char)0 + "")) {
				s.append(prefix + x + ": \"" + temp + "\" (NULL TERMINATED)\n");
			}
			else {
				s.append(prefix + x + ": \"" + temp + "\"\n");
			}
			x += temp.length() + 1;
		}
	}
}
