package crystalpalace.coff;

import crystalpalace.util.*;
import java.util.*;

public class Symbol extends COFFPart implements Comparable {
	protected int    entryNumber;
	protected byte[] Name;
	protected long   Value;
	protected int    SectionNumber;
	protected int    Type;

	public Symbol(int x, COFFObject o, ByteWalker walker) {
		super(o);

		entryNumber          = x;

		Name                 = walker.popBytes(8);	/* the interpretation of this field is something else for later */
		Value                = walker.readInt();
		SectionNumber        = walker.readShort();
		Type                 = walker.readShort();
		walker.popBytes(2);
	}

	public boolean is(int x) {
		return x == entryNumber;
	}

	/*
	 * Name is:
	 *  (1) a zero-terminated string, up to 8 bytes, OR
	 *  (2) int x = 0, int y = offset into string table with name
	 */
	public String getName() {
		int x;
		int y;

		ByteWalker walker = new ByteWalker(Name);
		walker.Mark();
		x = walker.readInt();
		y = walker.readInt();
		walker.Return();

		if (x == 0 && y == 0) {
			return "";
		}
		else if (x == 0) {
			return getObject().getStringAt(y);
		}
		else {
			return walker.readStringA(8);
		}
	}

	public Section getSection() {
		return getObject().getSection(SectionNumber);
	}

	public long getValue() {
		return Value;
	}

	public int compareTo(Object other) {
		return (int)getValue() - (int)((Symbol)other).getValue();
	}


	public long estimateSize() {
		Iterator i    = getObject().getSymbolsForSection(SectionNumber).iterator();
		while (i.hasNext()) {
			Symbol temp = (Symbol)i.next();

			/* if it's the NEXT symbol, then it's offset is the end of our data. */
			if (temp.getValue() > getValue())
				return temp.getValue() - getValue();
		}

		/* And, if we are the LAST symbol in our section, then the section size is the end of our data */
		if (getSection() == null)
			return 0;

		return getSection().getRawData().length - getValue();
	}

	public void toString(String prefix, StringBuffer s) {
		s.append(prefix + "Name:                 " + getName() + "\n");
		s.append(prefix + "Value:                " + Value + " ("+CrystalUtils.toHex(Value)+")\n");
		s.append(prefix + "SectionNumber:        " + SectionNumber + "\n");
		s.append(prefix + "Type:                 " + Type + "\n");
		//s.append(prefix + "Size (est.):          " + estimateSize() + "\n");
	}
}
