package crystalpalace.export;

import crystalpalace.coff.*;
import crystalpalace.util.*;

import java.util.*;
import java.nio.*;

/*
 * What do we do here? We pack in values, resolve symbols, blah blah blah
 */
public class ExportPIC extends ExportObject {
	public ExportPIC(COFFObject object) {
		super(object);
	}

	public void addSections() {
		addSectionRaw(".text");
	}

	protected void processRelocation(String section, Relocation r) {
		if (!hasOffset( r.getSymbolName() )) {
			if ( ".rdata".equals(r.getSymbolName()) ) {
				throw new RuntimeException("Can't process relocation for " + r.toSimpleString() + ". Did you try to use a \"string literal\" from PIC?");
			}

			throw new RuntimeException("Can't process relocation for " + r.toSimpleString());
		}

		if (r.is("x64", Relocation.IMAGE_REL_AMD64_REL32)) {
			Logger.print_info("Relocation r: " + r);

			long reladdress = getOffset( r.getSymbolName() ) - ( r.getVirtualAddress() + 4 );
			int  offset     = getBuffer(section).getInt((int)r.getVirtualAddress());

			getBuffer(section).putInt((int)r.getVirtualAddress(), (int)reladdress + offset);
		}
		/*
		 * NOTE: this is NOT how this relocation is expected to be handled in any other sane way.
		 * But, we're using it as a hack here. We're giving the x86 code the offset to the symbol,
		 * from a specific instruction. It's up to the program to combine this with some other offset
		 * (e.g., the address of the preceeding instruction, obtained with caller(). This is necessary
		 * because x86 does not have a means of loading something from an indirect address.
		 */
		else if (r.is("x86", Relocation.IMAGE_REL_I386_DIR32)) {
			long reladdress = getOffset( r.getSymbolName() ) - ( r.getVirtualAddress() + 4 );
			int  offset     = getBuffer(section).getInt((int)r.getVirtualAddress());

			getBuffer(section).putInt((int)r.getVirtualAddress(), (int)reladdress + offset);
		}
		else {
			throw new RuntimeException("Can't offline process relocation " + r);
		}
	}

	public boolean isPIC() {
		return true;
	}

	public static void main(String args[]) {
		if (args.length != 2) {
			CrystalUtils.print_error("./picbuild [/path/to/file.o] [out.bin]");
			return;
		}

		try {
			ExportPIC p = new COFFParser(CrystalUtils.readFromFile(args[0])).parse().export();
			CrystalUtils.writeToFile(args[1], p.export());
		}
		catch (Exception ex) {
			CrystalUtils.handleException(ex);
		}
	}
}
